//******************************************************************************
// Generic Test for the miniSensing Test Board 
//
//
// Tested Platform: mini Sensing Board PIC32
// Tested MCU: PIC32MX150F128D
// Tested Compiler: XC32 4.35
//******************************************************************************

#include "xc.h"

#define SYSTEM_CLOCK 32000000UL

#include "LTlib.h"

#include "HW_mini_Sensing_PIC_Board.h"

#include "module_IO.h"  
#include "module_IO.c" 

#include "LTlib_delay.h"
#include "LTlib_delay.c"

#include "module_I2C.h"
#include "module_I2C.c"

#include "OPT3001.h"
#include "OPT3001.c"

#include "BME280.h"
#include "BME280.c"

#include "HDC2080.h"
#include "HDC2080.c"

#include "LSM6DSM.h"
#include "LSM6DSM.c"

#define ADDRESS_OPT      0x44 //A0GND
#define ADDRESS_HUMIDITY 0x40
#define ADDRESS_PRESSURE 0x76 
#define ADDRESS_GYRO_ACC 0x6A

#define RUN_TEST_OPT  
#define RUN_TEST_HUMIDITY  
#define RUN_TEST_PRESSURE
#define RUN_TEST_GYRO_ACC
 

#define TIME_ON 500u
#define TIME_BLINK 500u
#define BEEP_COUNT 1000u

void all_LED_OFF (void);
void all_LED_ON (void);
void LED_ON_sequence (void);

int main (void){
    uint8_t device_ID = 0;
    uint16_t device_IDx16 = 0;
    uint16_t counter = 0x00;
    
    IO_set_all_ports_as_inputs ();
    delay_set_quartz(16);
    
    
    //Set LED as output
    LED_1_DIR = SET_AS_OUTPUT;
    LED_2_DIR = SET_AS_OUTPUT;
        
    //Output Buzzer
    BUZZER_DIR = SET_AS_OUTPUT;
    
    //Open Drain as output 
    OPEN_DRAIN_DIR = SET_AS_OUTPUT;
        
    //Servo
    SERVO_DIR = SET_AS_OUTPUT;
    
        
    //Initialization
    LED_1 = LED_OFF;
    LED_2 = LED_ON;
    
    BUZZER_OUT = OFF_STATE;
    OPEN_DRAIN_OUT = OFF_STATE;
    SERVO_OUT = OFF_STATE;
    
    //************************************
    // Buzzer Test
    //************************************
    for (counter = 0; counter <BEEP_COUNT; counter++ ){
        delay_us (240);
        BUZZER_OUT = (BUZZER_OUT^0x01);  
    }


    //************************************
    P1_DIR = SET_AS_OUTPUT;
    P2_DIR = SET_AS_OUTPUT;
    P3_DIR = SET_AS_OUTPUT;
    P4_DIR = SET_AS_OUTPUT;
    P5_DIR = SET_AS_OUTPUT;
    P6_DIR = SET_AS_OUTPUT;
    P7_DIR = SET_AS_OUTPUT;
    P8_DIR = SET_AS_OUTPUT;  
    P9_DIR = SET_AS_OUTPUT;
    P10_DIR = SET_AS_OUTPUT;
    P11_DIR = SET_AS_OUTPUT;
    P12_DIR = SET_AS_OUTPUT;
    P13_DIR = SET_AS_OUTPUT;
    P14_DIR = SET_AS_OUTPUT;
    P15_DIR = SET_AS_OUTPUT;
    P16_DIR = SET_AS_OUTPUT;
    P17_DIR = SET_AS_OUTPUT;
    P18_DIR = SET_AS_OUTPUT;
    P19_DIR = SET_AS_OUTPUT;
    P20_DIR = SET_AS_OUTPUT;

        
    all_LED_OFF ();
    
    LED_ON_sequence ();
    
    //Check Sensors
    
#ifdef RUN_TEST_OPT  

    //Light OPT
    OPT3001_initialize(400);
    OPT3001_get_device_ID (ADDRESS_OPT, &device_IDx16);
    
    if (device_IDx16 == OPT3001_DEVICE_ID) {
        P20_OUT = LED_ON;
    } else {
        P20_OUT = LED_OFF;
    }
#endif      
    
#ifdef RUN_TEST_HUMIDITY  

    //Humidity
    HDC2080_initialize(ADDRESS_HUMIDITY, 400); 
    HDC2080_get_device_ID (ADDRESS_HUMIDITY, &device_IDx16);
        
    if (device_IDx16 == HDC2080_DEVICE_ID) {
        P19_OUT = LED_ON;
    } else {
        P19_OUT = LED_OFF;
    }
#endif  
    
#ifdef RUN_TEST_PRESSURE

    //Pressure
    BME280_initialize (ADDRESS_PRESSURE, 400);
    BME280_get_device_ID (ADDRESS_PRESSURE, &device_ID);
    
    if (device_ID == BME280_DEVICE_ID) {
        P18_OUT = LED_ON;
    } else {
        P18_OUT = LED_OFF;
    }
#endif      
    
#ifdef RUN_TEST_GYRO_ACC
    //Acc. Gyro
    LSM6DSM_initialize (ADDRESS_GYRO_ACC, 400);
    LSM6DSM_get_device_ID (ADDRESS_GYRO_ACC, &device_ID); 
    
    if (device_ID == LSM6DSM_DEVICE_ID) {
        P17_OUT = LED_ON;
    } else {
        P17_OUT = LED_OFF;
    }
#endif    

while (1){
    delay_ms (TIME_BLINK);
    LED_1 ^= 0x01;
    LED_2 ^= 0x01;     
    
    OPEN_DRAIN_OUT ^= ON_STATE;
    SERVO_OUT ^= ON_STATE;
    
    P20_OUT ^= LED_ON;
    P19_OUT ^= LED_ON;
    P18_OUT ^= LED_ON;
    P17_OUT ^= LED_ON;
    
    //************************************
    P8_OUT ^= LED_ON;
    P9_OUT ^= LED_ON;
    P10_OUT ^= LED_ON;
    P11_OUT ^= LED_ON;
    P12_OUT ^= LED_ON;
    P13_OUT ^= LED_ON;
    
    P14_OUT ^= LED_ON;
    P15_OUT ^= LED_ON;
    P16_OUT ^= LED_ON;
    
    //************************************
    
    P7_OUT ^= LED_ON;
    P6_OUT ^= LED_ON;
    P5_OUT ^= LED_ON;
    P4_OUT ^= LED_ON;
    P3_OUT ^= LED_ON;
    P2_OUT ^= LED_ON;
    P1_OUT ^= LED_ON;
    
}
    

}

//******************************************************************************
void all_LED_OFF (void){
    
    P20_OUT = LED_OFF;
    P19_OUT = LED_OFF;
    P18_OUT = LED_OFF;
    P17_OUT = LED_OFF;
    
    //************************************
    P8_OUT = LED_OFF;
    P9_OUT = LED_OFF;
    P10_OUT = LED_OFF;
    P11_OUT = LED_OFF;
    P12_OUT = LED_OFF;
    P13_OUT = LED_OFF;
    
    P14_OUT = LED_OFF;
    P15_OUT = LED_OFF;
    P16_OUT = LED_OFF;
    
    //************************************
    
    P7_OUT = LED_OFF;
    P6_OUT = LED_OFF;
    P5_OUT = LED_OFF;
    P4_OUT = LED_OFF;
    P3_OUT = LED_OFF;
    P2_OUT = LED_OFF;
    P1_OUT = LED_OFF;
}

//******************************************************************************
void all_LED_ON (void){
    
    P20_OUT = LED_ON;
    P19_OUT = LED_ON;
    P18_OUT = LED_ON;
    P17_OUT = LED_ON;
    
    //************************************
    P8_OUT = LED_ON;
    P9_OUT = LED_ON;
    P10_OUT = LED_ON;
    P11_OUT = LED_ON;
    P12_OUT = LED_ON;
    P13_OUT = LED_ON;
    
    P14_OUT = LED_ON;
    P15_OUT = LED_ON;
    P16_OUT = LED_ON;
    
    //************************************
    
    P7_OUT = LED_ON;
    P6_OUT = LED_ON;
    P5_OUT = LED_ON;
    P4_OUT = LED_ON;
    P3_OUT = LED_ON;
    P2_OUT = LED_ON;
    P1_OUT = LED_ON;
}


//******************************************************************************
void LED_ON_sequence (void){
    
    P20_OUT = LED_ON;    
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P19_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P18_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P17_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    //************************************
    P8_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P9_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P10_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P11_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P12_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P13_OUT  = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    
    P14_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P15_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P16_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    //************************************
    
    P7_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P6_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P5_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P4_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P3_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P2_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
    
    P1_OUT = LED_ON;
    delay_ms(TIME_ON);
    all_LED_OFF ();
        
}
